-- Espionage Overview Master Process File
-- Author: sman
-- DateCreated: 01/30/18 5:06:34 PM
--------------------------------------------------------------

include("IconSupport")
include("InstanceManager")

include("00_Globals.lua")
include("05_Instantiate_Globals.lua")														-- Instantiates date from the global tables into working data stores
include("60_Mission_Support_Functions.lua")													-- Various support functions used by all of the following include files/this file
include("20_Process_Mission_Results.lua")													-- Process the results of a CO mission
include("30_Process_International_Responses.lua")											-- Determines international community responses for all covert operations
include("50_Process_Recon_Results.lua")														-- Builds the Recon Report popup/notification for the human player only; AI doesn't use Recon mission
include("25_Process_AI_Mission_Results.lua")												-- Performs AI missions, in a similar way that the human player does, without UI features

local g_ItemManager = InstanceManager:new( "COMenuInstance", "COMenuButton", Controls.COMenuStack )
local g_sSelectedAgentName
local g_iSelectedAgentRank
local g_pSelectedCity
local g_iSelectedMission

-- ===============================================================================================
--		Support Functions




-- ===============================================================================================
--		Covert Ops Mission Selection Popup 

function DisplayCOMenuPopup(sSelectedAgentName, iSelectedAgentRank, pSelectedCity, g_iMissionCost)		-- Function that builds the popup and displays it	
	if (not sSelectedAgentName) or (not pSelectedCity) then 
		print("Inside DisplayCOMenuPopup. No agent name or no city pointer error!")
		return
	end
	
	local pPlayer = Players[Game.GetActivePlayer()]											-- Get current player; ok to use GetActivePlayer as only the human player can use this popup	
	local iCurrentEra = pPlayer:GetCurrentEra()												-- Get Active (human) player's current Era
	local iAttackingPlayerGold = pPlayer:GetGold()											-- Get total gold target civ has on hand	
	local pTargetPlayer = Players[pSelectedCity:GetOwner()]									-- Get the target city owner's pointer
	
	g_pSelectedCity = pSelectedCity															-- Assign value to target city global pointer
	g_sSelectedAgentName = sSelectedAgentName												-- Assign value to global spy name
	g_iSelectedAgentRank = iSelectedAgentRank												-- Assign value to global spy rank (1..3)
	g_ItemManager:ResetInstances()
		
	local bSelectedCityIsCapital = false													-- Is target city (selected city) a capital?, default to false
	if pSelectedCity:IsCapital() then bSelectedCityIsCapital = true end						-- If target city is capital, set flag to true
	local bSelectedCityIsCityState = false													-- Is target city (selected city) a city state?, default to false
	if pTargetPlayer:IsMinorCiv() then bSelectedCityIsCityState = true end					-- If target city is city state, set flag to true
		
	local sTitleString = Locale.ConvertTextKey("TXT_KEY_COVERT_OPS_BODY", g_sSelectedAgentName, g_pSelectedCity:GetName())
	Controls.COMenuMessage:LocalizeAndSetText(sTitleString)

	for iNum = 1, #g_MissionList do															-- Cycle through all missions
		if (g_MissionList[iNum][8] <= iCurrentEra) then										-- If Mission #iNum's "level" is LTE to civ's current Era, allow it in the menu
			
			g_iMissionCost = GetMissionCost(pPlayer, iNum, pTargetPlayer, pSelectedCity)			
			if ( g_iMissionCost <= iAttackingPlayerGold ) then 

				if  (not bSelectedCityIsCapital) or													-- For non-capital cities, all missions valid, then...	
						((g_ValidMissions[iNum][CAPITAL_CITY] == YES) and (bSelectedCityIsCapital) and (not bSelectedCityIsCityState))  or	-- For capital city: mission is valid, city is cap, city is not city state, then
						((g_ValidMissions[iNum][CITY_STATE] == YES) and (bSelectedCityIsCityState)  )  then		-- For city state: mission is valid, city is city state, then...

					local controlTable = g_ItemManager:GetInstance()		

					--IconHookup( iNum + 15, 64, "SMAN_COVERT_OPS_ACTION_ATLAS", controlTable.Icon64 )	-- Silver Icon at top of popup
					IconHookup( iNum - 1, 64, "SMAN_COVERT_OPS_ACTION_ATLAS", controlTable.Icon64 )	-- Gold Icon at top of popup

					local keyMissionTitleString = "TXT_KEY_COVERT_OPS_MISSION_TITLE_" .. iNum		-- Get the correct text strings for this mission type
					local keyMissionBodyString = "TXT_KEY_COVERT_OPS_MISSION_BODY_" .. iNum
		
					controlTable.COMenuLabel:LocalizeAndSetText(keyMissionTitleString)
					controlTable.COMenuButton:SetToolTipString(Locale.ConvertTextKey(keyMissionBodyString, g_MissionList[iNum][2], g_MissionList[iNum][4], g_iMissionCost))
					controlTable.COMenuButton:SetDisabled(false)	

					local selectionAnim = controlTable.COMenuSelectionAnim
					controlTable.COMenuButton:SetVoid1(iNum)
					controlTable.COMenuButton:RegisterCallback(Mouse.eLClick, function()  g_iSelectedMission = iNum  DisplayCOMenuPopup(sSelectedAgentName, iSelectedAgentRank, g_pSelectedCity, g_iMissionCost)  end)		

					if g_iSelectedMission then
						if g_iSelectedMission == iNum then
								controlTable.COMenuSelectionAnim:SetHide(false)
						else
								controlTable.COMenuSelectionAnim:SetHide(true)
						end
					else
							controlTable.COMenuSelectionAnim:SetHide(true)
					end
				end
			end
		end
	end	
	
	Controls.COMenuStack:CalculateSize()
	Controls.COMenuStack:ReprocessAnchoring()
	Controls.COMenuPanel:CalculateInternalSize()	
	--Controls.COMenuPanel:SetScrollValue(0)
	Controls.ConfirmButton:SetDisabled(true)
	Controls.COMenuPopup:SetHide(false)

	if g_iSelectedMission then
		Controls.ConfirmButton:SetDisabled(false)
	end	
end
LuaEvents.CovertMissionPopup.Add( DisplayCOMenuPopup )


function SetNextHumanCOGameTurn()	
	local iNextCOTurn = g_Human_Last_Turn_CO_Launched + g_Human_Minimum_Turn_Interval + math.random(-1, 3)		
	--print("Next human player CO turn: " .. iNextCOTurn)
	g_Human_Next_CO_Launch_Turn = iNextCOTurn
end


function OnConfirm(playerID)																-- Process actions once the player has selected a mission	
	g_iActiveMission = g_iSelectedMission
	g_iSelectedMission = ABORT_MISSION														-- Sets the default menu position to "Abort Mission" for the next time the popup unhides
	Controls.COMenuPanel:SetScrollValue(0)
	Controls.COMenuPopup:SetHide(true)
	   
	--print("========> Selected mission: " .. g_iActiveMission .. "   Mission name: " .. g_MissionList[g_iActiveMission][1])
	ProcessMission (g_iActiveMission, g_pSelectedCity)										-- Process the selected mission	
		
	if g_iActiveMission == ABORT_MISSION then
		Controls.LaunchCOButton:SetDisabled(false)											-- Disable the launch button
	else
		Controls.LaunchCOButton:SetDisabled(true)											-- Disable the launch button
		SetNextHumanCOGameTurn()
	end	
end
Controls.ConfirmButton:RegisterCallback( Mouse.eLClick, OnConfirm )



-- ====================================================================================================================================
--		Recon/Mission results popup

local g_ReconReportManager = InstanceManager:new("ReconReport", "ReconReportLabel", Controls.ReconReportStack)

function ProcessReconPopup (sReportTitle, sReportHeader, sReconReport, iTopIcon)	
	
	g_ReconReportManager:ResetInstances()
		
	IconHookup( iTopIcon + 33, 80, "SMAN_COVERT_OPS_ACTION_ATLAS", Controls.ReconReportTopIcon )	-- Icon at top of popup, 1=gold, 2=silve, 3=red
		
	Controls.ReconReportTitle:LocalizeAndSetText("TXT_KEY_COVERT_OPS_RECON_REPORT_TITLE", sReportTitle)		-- Report title (green bar)
	Controls.ReconReportHeader:LocalizeAndSetText("TXT_KEY_COVERT_OPS_RECON_REPORT_HEADER", sReportHeader)		-- Set value for header of report
	
	local instance = g_ReconReportManager:GetInstance()
	instance.ReconReportLabel:SetText(sReconReport)											-- Set value for body of report

	Controls.ReconReportStack:CalculateSize()
	Controls.ReconReportStack:ReprocessAnchoring()
	Controls.ReconReportPanel:CalculateInternalSize()
	Controls.ReconReportPanel:SetScrollValue(0)
		
	Controls.ReconReportPopup:SetHide(false)
end
LuaEvents.ProcessReconReport.Add( ProcessReconPopup )


function OnReconReportOK()
	Controls.ReconReportPopup:SetHide(true)
end
Controls.ReconReportOK:RegisterCallback(Mouse.eLClick, OnReconReportOK)

